# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_helper/api/events.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grpc_helper.api import common_pb2 as grpc__helper_dot_api_dot_common__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
    name="grpc_helper/api/events.proto",
    package="",
    syntax="proto3",
    serialized_options=b"\n\017grpc_helper.apiP\001",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1cgrpc_helper/api/events.proto\x1a\x1cgrpc_helper/api/common.proto",\n\rEventProperty\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t"9\n\x05\x45vent\x12\x0c\n\x04name\x18\x01 \x01(\t\x12"\n\nproperties\x18\x02 \x03(\x0b\x32\x0e.EventProperty"K\n\x0b\x45ventStatus\x12\x12\n\x01r\x18\x01 \x01(\x0b\x32\x07.Result\x12\x15\n\x05\x65vent\x18\x02 \x01(\x0b\x32\x06.Event\x12\x11\n\tclient_id\x18\x03 \x01(\r"/\n\x0b\x45ventFilter\x12\r\n\x05names\x18\x01 \x03(\t\x12\x11\n\tclient_id\x18\x02 \x01(\r"#\n\x0e\x45ventInterrupt\x12\x11\n\tclient_id\x18\x01 \x01(\r":\n\x10\x45ventQueueStatus\x12\x12\n\x01r\x18\x01 \x01(\x0b\x32\x07.Result\x12\x12\n\nclient_ids\x18\x02 \x03(\r*X\n\x0f\x45ventApiVersion\x12\x15\n\x11\x45VENT_API_UNKNOWN\x10\x00\x12\x17\n\x13\x45VENT_API_SUPPORTED\x10\x01\x12\x15\n\x11\x45VENT_API_CURRENT\x10\x02\x32\xa8\x01\n\x0c\x45ventService\x12&\n\x06listen\x12\x0c.EventFilter\x1a\x0c.EventStatus0\x01\x12+\n\tinterrupt\x12\x0f.EventInterrupt\x1a\r.ResultStatus\x12\x1d\n\x04send\x12\x06.Event\x1a\r.ResultStatus\x12$\n\x07inspect\x12\x06.Empty\x1a\x11.EventQueueStatusB\x13\n\x0fgrpc_helper.apiP\x01\x62\x06proto3',
    dependencies=[
        grpc__helper_dot_api_dot_common__pb2.DESCRIPTOR,
    ],
)

_EVENTAPIVERSION = _descriptor.EnumDescriptor(
    name="EventApiVersion",
    full_name="EventApiVersion",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="EVENT_API_UNKNOWN", index=0, number=0, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="EVENT_API_SUPPORTED", index=1, number=1, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
        _descriptor.EnumValueDescriptor(
            name="EVENT_API_CURRENT", index=2, number=2, serialized_options=None, type=None, create_key=_descriptor._internal_create_key
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=390,
    serialized_end=478,
)
_sym_db.RegisterEnumDescriptor(_EVENTAPIVERSION)

EventApiVersion = enum_type_wrapper.EnumTypeWrapper(_EVENTAPIVERSION)
EVENT_API_UNKNOWN = 0
EVENT_API_SUPPORTED = 1
EVENT_API_CURRENT = 2


_EVENTPROPERTY = _descriptor.Descriptor(
    name="EventProperty",
    full_name="EventProperty",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="EventProperty.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="value",
            full_name="EventProperty.value",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=62,
    serialized_end=106,
)


_EVENT = _descriptor.Descriptor(
    name="Event",
    full_name="Event",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="Event.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="properties",
            full_name="Event.properties",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=108,
    serialized_end=165,
)


_EVENTSTATUS = _descriptor.Descriptor(
    name="EventStatus",
    full_name="EventStatus",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="r",
            full_name="EventStatus.r",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="event",
            full_name="EventStatus.event",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="client_id",
            full_name="EventStatus.client_id",
            index=2,
            number=3,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=167,
    serialized_end=242,
)


_EVENTFILTER = _descriptor.Descriptor(
    name="EventFilter",
    full_name="EventFilter",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="names",
            full_name="EventFilter.names",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="client_id",
            full_name="EventFilter.client_id",
            index=1,
            number=2,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=244,
    serialized_end=291,
)


_EVENTINTERRUPT = _descriptor.Descriptor(
    name="EventInterrupt",
    full_name="EventInterrupt",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="client_id",
            full_name="EventInterrupt.client_id",
            index=0,
            number=1,
            type=13,
            cpp_type=3,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=293,
    serialized_end=328,
)


_EVENTQUEUESTATUS = _descriptor.Descriptor(
    name="EventQueueStatus",
    full_name="EventQueueStatus",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="r",
            full_name="EventQueueStatus.r",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="client_ids",
            full_name="EventQueueStatus.client_ids",
            index=1,
            number=2,
            type=13,
            cpp_type=3,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=330,
    serialized_end=388,
)

_EVENT.fields_by_name["properties"].message_type = _EVENTPROPERTY
_EVENTSTATUS.fields_by_name["r"].message_type = grpc__helper_dot_api_dot_common__pb2._RESULT
_EVENTSTATUS.fields_by_name["event"].message_type = _EVENT
_EVENTQUEUESTATUS.fields_by_name["r"].message_type = grpc__helper_dot_api_dot_common__pb2._RESULT
DESCRIPTOR.message_types_by_name["EventProperty"] = _EVENTPROPERTY
DESCRIPTOR.message_types_by_name["Event"] = _EVENT
DESCRIPTOR.message_types_by_name["EventStatus"] = _EVENTSTATUS
DESCRIPTOR.message_types_by_name["EventFilter"] = _EVENTFILTER
DESCRIPTOR.message_types_by_name["EventInterrupt"] = _EVENTINTERRUPT
DESCRIPTOR.message_types_by_name["EventQueueStatus"] = _EVENTQUEUESTATUS
DESCRIPTOR.enum_types_by_name["EventApiVersion"] = _EVENTAPIVERSION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EventProperty = _reflection.GeneratedProtocolMessageType(
    "EventProperty",
    (_message.Message,),
    {
        "DESCRIPTOR": _EVENTPROPERTY,
        "__module__": "grpc_helper.api.events_pb2"
        # @@protoc_insertion_point(class_scope:EventProperty)
    },
)
_sym_db.RegisterMessage(EventProperty)

Event = _reflection.GeneratedProtocolMessageType(
    "Event",
    (_message.Message,),
    {
        "DESCRIPTOR": _EVENT,
        "__module__": "grpc_helper.api.events_pb2"
        # @@protoc_insertion_point(class_scope:Event)
    },
)
_sym_db.RegisterMessage(Event)

EventStatus = _reflection.GeneratedProtocolMessageType(
    "EventStatus",
    (_message.Message,),
    {
        "DESCRIPTOR": _EVENTSTATUS,
        "__module__": "grpc_helper.api.events_pb2"
        # @@protoc_insertion_point(class_scope:EventStatus)
    },
)
_sym_db.RegisterMessage(EventStatus)

EventFilter = _reflection.GeneratedProtocolMessageType(
    "EventFilter",
    (_message.Message,),
    {
        "DESCRIPTOR": _EVENTFILTER,
        "__module__": "grpc_helper.api.events_pb2"
        # @@protoc_insertion_point(class_scope:EventFilter)
    },
)
_sym_db.RegisterMessage(EventFilter)

EventInterrupt = _reflection.GeneratedProtocolMessageType(
    "EventInterrupt",
    (_message.Message,),
    {
        "DESCRIPTOR": _EVENTINTERRUPT,
        "__module__": "grpc_helper.api.events_pb2"
        # @@protoc_insertion_point(class_scope:EventInterrupt)
    },
)
_sym_db.RegisterMessage(EventInterrupt)

EventQueueStatus = _reflection.GeneratedProtocolMessageType(
    "EventQueueStatus",
    (_message.Message,),
    {
        "DESCRIPTOR": _EVENTQUEUESTATUS,
        "__module__": "grpc_helper.api.events_pb2"
        # @@protoc_insertion_point(class_scope:EventQueueStatus)
    },
)
_sym_db.RegisterMessage(EventQueueStatus)


DESCRIPTOR._options = None

_EVENTSERVICE = _descriptor.ServiceDescriptor(
    name="EventService",
    full_name="EventService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=481,
    serialized_end=649,
    methods=[
        _descriptor.MethodDescriptor(
            name="listen",
            full_name="EventService.listen",
            index=0,
            containing_service=None,
            input_type=_EVENTFILTER,
            output_type=_EVENTSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="interrupt",
            full_name="EventService.interrupt",
            index=1,
            containing_service=None,
            input_type=_EVENTINTERRUPT,
            output_type=grpc__helper_dot_api_dot_common__pb2._RESULTSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="send",
            full_name="EventService.send",
            index=2,
            containing_service=None,
            input_type=_EVENT,
            output_type=grpc__helper_dot_api_dot_common__pb2._RESULTSTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="inspect",
            full_name="EventService.inspect",
            index=3,
            containing_service=None,
            input_type=grpc__helper_dot_api_dot_common__pb2._EMPTY,
            output_type=_EVENTQUEUESTATUS,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_EVENTSERVICE)

DESCRIPTOR.services_by_name["EventService"] = _EVENTSERVICE

# @@protoc_insertion_point(module_scope)
